AdiAccelerometer

Struct AdiAccelerometer 

pub struct AdiAccelerometer { /* private fields */ }
Available on crate feature devices only.
A single axis connection to the 3-axis analog accelerometer.

Implementations

Source

impl AdiAccelerometer

pub fn new(port: AdiPort, sensitivity: Sensitivity) -> AdiAccelerometer

Create a new accelerometer from an AdiPort.

pub const fn sensitivity(&self) -> Sensitivity

Returns the configured sensitivity of the ADI accelerometer device.

pub const fn max_acceleration(&self) -> f64

Returns the maximum acceleration measurement supported by the current Sensitivity jumper.

pub fn acceleration(&self) -> Result<f64, PortError>

Returns the current acceleration measurement for this axis in g (~9.8 m/s/s).

Errors

These errors are only returned if the device is plugged into an AdiExpander.

pub fn raw_acceleration(&self) -> Result<u16, PortError>

Returns the raw acceleration reading from [0, 4096]. This represents an ADC-converted analog input from 0-5V.

For example, when on high sensitivity a value of 4096 would represent a reading of 6g (Sensitivity::HIGH_MAX_ACCELERATION). When on low acceleration, this same value would instead represent a 2g reading (Sensitivity::LOW_MAX_ACCELERATION).

Errors

These errors are only returned if the device is plugged into an AdiExpander.