pub fn code_signature() -> CodeSignatureAvailable on crate feature
core only.Expand description
Returns the code signature of the currently running program.
pub fn code_signature() -> CodeSignaturecore only.Returns the code signature of the currently running program.