pub struct VisionObject {
pub source: DetectionSource,
pub width: u16,
pub height: u16,
pub offset: Point2<u16>,
pub center: Point2<u16>,
pub angle: Angle,
}Available on crate feature
devices only.Expand description
A detected vision object.
This struct contains metadata about objects detected by the vision sensor. Objects are detected
by calling VisionSensor::objects after adding signatures and color codes to the sensor.
Fields
source: DetectionSourceThe ID of the signature or color code used to detect this object.
width: u16The width of the detected object’s bounding box in pixels.
height: u16The height of the detected object’s bounding box in pixels.
offset: Point2<u16>The top-left coordinate of the detected object relative to the top-left of the camera’s field of view.
center: Point2<u16>The center coordinate of the detected object relative to the top-left of the camera’s field of view.
angle: AngleThe approximate degrees of rotation of the detected object’s bounding box.